import unicodedata
import urllib.parse
import bs4


class StringContext:
  def __init__(self, url, str):
    self.url = url
    self.str = str

  def select(self, selector):
    raise TypeError('Cannot run selector on a StringContext.')

  def text(self):
    return self.str

  def __str__(self):
    return self.str


class HtmlContext:
  def __init__(self, url, str_or_element):
    self.url = url
    self.element = str_or_element

    if isinstance(str_or_element, str):
      self.element = self.parse(self.url, self.element)

  def select(self, selector):
    # Determine whether selector is using ::attr or ::text pseudo-attribute.
    selector_type = None
    if selector.endswith('::text'):
      selector_type = 'text'
      selector = selector.rstrip('::text')
    elif selector.find('::attr') > 0:
      # Use attribute as a selector type.
      selector_type = selector[selector.find('::attr') + 7:-1]
      selector = selector[:selector.find('::attr')]

    selected = []
    for element in self.element.select(selector):
      if selector_type is None:
        selected.append(HtmlContext(self.url, element))
      elif selector_type == 'text':
        selected.append(
          StringContext(self.url, HtmlContext(self.url, element).text()))
      else:
        selected.append(StringContext(self.url, element[selector_type]))
    return selected

  def text(self):
    """Convenience method to extract text from HTML element tree. Performs
      stripping and canonicalization of UTF-8 characters (e.g. '/xa0' to ' ')."""
    return unicodedata.normalize('NFKC', self.element.text.strip())

  def __str__(self):
    """Convert HTML element to raw html string including element tags."""
    return str(self.element)

  @staticmethod
  def parse(url, html_string):
    doc = bs4.BeautifulSoup(html_string, features='html5lib')
    # Making links absolute is required to allow following.
    for tag in doc.findAll('a', href=True):
      tag['href'] = urllib.parse.urljoin(url, tag['href'])
    # Replacing <br> tags with \n, prevents text concatenating.
    for br in doc.findAll('br'):
      br.replace_with("\n")
    return doc



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['hypotonic']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.0.9',
 'beautifulsoup4>=4.7.1',
 'html5lib>=1.0.1',
 'more_itertools>=6.0.0']

setup_kwargs = {
    'name': 'hypotonic',
    'version': '0.0.11',
    'description': 'Fast asynchronous web scraper with minimalist API.',
    'long_description': "# Hypotonic\n\nFast asynchronous web scraper with minimalist API inspired by awesome [node-osmosis](https://github.com/rchipka/node-osmosis).\n\nHypotonic provides SQLAlchemy-like command chaining DSL to define HTML scrapers. Everything is executed asynchronously via `asyncio` and all dependencies are pure Python. Supports querying by CSS selectors with Scrapy's pseudo-attributes. XPath is not supported due to `libxml` requirement.\n\nHypotonic does not natively execute JavaScript on websites and it is recommended to use [prerender](https://prerender.com).\n\n## Installing\n\nHypotonic requires Python 3.6+.\n\n`pip install hypotonic`\n\n## Example\n\n```python\nfrom hypotonic import Hypotonic\n\ndata, errors = (\n  Hypotonic()\n    .get('http://books.toscrape.com/')\n    .paginate('.next a::attr(href)', 5)\n    .find('.product_pod h3')\n    .set('title')\n    .follow('a::attr(href)')\n    .set({'price': '.price_color',\n          'availability': 'p.availability'})\n    .data()\n)\n```\n",
    'author': 'Martin Scavnicky',
    'author_email': 'martin.scavnicky@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/mscavnicky/hypotonic',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
